@@ -25,6 +25,7 @@ use serde::Serialize;
 use super::chunk_store::{ReadChunk, StoreReader, StoreWriter, WriteChunk};
 use super::handle::{chunk_hash, ContentId, Extent, ObjectHandle, ObjectStats};
 use super::state::{ExtentLocation, ObjectState, RepoState, SeekPosition};
+use crate::repo::ObjectId;
 
 pub struct ObjectStore {
     repo_state: Arc<RwLock<RepoState>>,
@@ -86,10 +87,6 @@ pub struct ObjectReaderGuard<'a> {
 }
 
 impl<'a> ObjectReaderGuard<'a> {
-    pub fn info(&self) -> ObjectInfo {
-        ObjectInfo::new(&self.repo_state, &self.object_state, &self.handle)
-    }
-
     pub fn reader(&mut self) -> ObjectReader {
         ObjectReader::new(&self.repo_state, &mut self.object_state, &self.handle)
     }
@@ -102,14 +99,6 @@ pub struct ObjectWriterGuard<'a> {
 }
 
 impl<'a> ObjectWriterGuard<'a> {
-    pub fn info(&self) -> ObjectInfo {
-        ObjectInfo::new(&self.repo_state, &self.object_state, &self.handle)
-    }
-
-    pub fn reader(&mut self) -> ObjectReader {
-        ObjectReader::new(&self.repo_state, &mut self.object_state, &self.handle)
-    }
-
     pub fn writer(&mut self) -> ObjectWriter {
         ObjectWriter::new(
             &mut self.repo_state,
@@ -147,6 +136,11 @@ impl<'a> ObjectInfo<'a> {
         Ok(self.handle.size())
     }
 
+    /// Return an `ObjectId` representing the identity of the object.
+    pub fn object_id(&self) -> ObjectId {
+        ObjectId::new(self.repo_state.metadata.id, self.handle.id)
+    }
+
     /// Return a `ContentId` representing the contents of the object.
     pub fn content_id(&self) -> crate::Result<ContentId> {
         if self.object_state.transaction_lock.is_some() {
