@@ -15,26 +15,26 @@
  */
 
 use std::collections::HashMap;
+use std::fmt::{Debug, Formatter};
 use std::sync::{Arc, Mutex, RwLock};
 
 use hex_literal::hex;
-use once_cell::sync::Lazy;
 use rmp_serde::{from_read, to_vec};
 use secrecy::ExposeSecret;
 use uuid::Uuid;
 
-use crate::store::{DataStore, OpenStore};
+use crate::store::{BlockKey, DataStore, OpenStore};
 
 use super::chunking::Chunking;
 use super::compression::Compression;
 use super::config::RepoConfig;
 use super::encryption::{Encryption, EncryptionKey, KeySalt, ResourceLimit};
 use super::handle::HandleIdTable;
-use super::lock::LockTable;
-use super::metadata::{peek_info_store, Header, RepoId, RepoMetadata};
+use super::lock::{lock_store, LockTable};
+use super::metadata::{Header, RepoMetadata};
 use super::open_repo::OpenRepo;
 use super::packing::Packing;
-use super::repository::{KeyRepo, METADATA_BLOCK_ID, VERSION_BLOCK_ID};
+use super::repository::KeyRepo;
 use super::state::{InstanceId, RepoState};
 
 /// The default repository instance ID.
@@ -52,9 +52,6 @@ pub const DEFAULT_INSTANCE: InstanceId = InstanceId::new(Uuid::from_bytes(hex!(
 /// format.
 const VERSION_ID: Uuid = Uuid::from_bytes(hex!("44253e72 f08f 11eb a2a3 a701701f8601"));
 
-/// A table of locks on repositories.
-static REPO_LOCKS: Lazy<Mutex<LockTable<RepoId>>> = Lazy::new(|| Mutex::new(LockTable::new()));
-
 /// The mode to use to open a repository.
 #[derive(Debug, PartialEq, Eq, Hash, Clone, Copy)]
 pub enum OpenMode {
@@ -81,6 +78,7 @@ pub enum OpenMode {
 ///
 /// # Examples
 /// ```
+/// # #[cfg(all(feature = "encryption", feature = "compression"))] {
 /// use acid_store::repo::{OpenOptions, OpenMode, key::KeyRepo, Chunking, Compression, Encryption, Packing};
 /// use acid_store::store::MemoryConfig;
 ///
@@ -94,8 +92,10 @@ pub enum OpenMode {
 ///     .mode(OpenMode::Create)
 ///     .open(&store_config)
 ///     .unwrap();
+/// # }
 /// ```
 /// ```
+/// # #[cfg(all(feature = "encryption", feature = "compression"))] {
 /// use acid_store::repo::{OpenOptions, OpenMode, key::KeyRepo, Chunking, Compression, Encryption, Packing, RepoConfig};
 /// use acid_store::store::MemoryConfig;
 ///
@@ -112,27 +112,29 @@ pub enum OpenMode {
 ///     .mode(OpenMode::Create)
 ///     .open(&store_config)
 ///     .unwrap();
+/// # }
 /// ```
 ///
 /// [`new`]: crate::repo::OpenOptions::new
 /// [`open`]: crate::repo::OpenOptions::open
 /// [`OpenStore`]: crate::store::OpenStore
 /// [`OpenRepo`]: crate::repo::OpenRepo
-#[derive(Debug)]
-pub struct OpenOptions {
+pub struct OpenOptions<'a> {
     config: RepoConfig,
     mode: OpenMode,
-    password: Option<Vec<u8>>,
+    password: Option<&'a [u8]>,
     instance: InstanceId,
+    lock_context: &'a [u8],
+    lock_handler: Box<dyn FnMut(&[u8]) -> bool + 'a>,
 }
 
-impl Default for OpenOptions {
+impl<'a> Default for OpenOptions<'a> {
     fn default() -> Self {
         Self::new()
     }
 }
 
-impl OpenOptions {
+impl<'a> OpenOptions<'a> {
     /// Create a new `OpenOptions`.
     pub fn new() -> Self {
         Self {
@@ -140,6 +142,8 @@ impl OpenOptions {
             mode: OpenMode::Open,
             password: None,
             instance: DEFAULT_INSTANCE,
+            lock_context: &[],
+            lock_handler: Box::new(|_| false),
         }
     }
 
@@ -229,8 +233,53 @@ impl OpenOptions {
     /// Use the given `password`.
     ///
     /// This is required when encryption is enabled for the repository.
-    pub fn password(&mut self, password: &[u8]) -> &mut Self {
-        self.password = Some(password.to_vec());
+    pub fn password(&mut self, password: &'a [u8]) -> &mut Self {
+        self.password = Some(password);
+        self
+    }
+
+    /// Configure the behavior of repository locking.
+    ///
+    /// This method accepts a `context` which is associated with the lock on the repository once a
+    /// lock is acquired. If a lock's context is not specified, the context value of the acquired
+    /// lock will be empty. If encryption is enabled for the repository, the lock context is
+    /// encrypted. You can change the context value of the held lock once the repository is open
+    /// using [`Unlock::update_context`].
+    ///
+    /// This method also accepts a `handler` which is invoked if a lock is already held on the
+    /// repository. This lock handler is passed the context value of the existing lock. If `handler`
+    /// returns `true`, the existing lock will be removed and the repository will be opened. If
+    /// `handler` returns `false`, the existing lock will be respected and opening the repository
+    /// will fail. If a lock handler is not specified, an existing lock will always be respected.
+    ///
+    /// Opening a repository can still fail due to lock conflicts even if `handler` returns `true`
+    /// or is never called.
+    ///
+    /// **Removing an existing lock is potentially dangerous, as concurrent access to a repository
+    /// can cause data loss.**
+    ///
+    /// # Examples
+    ///
+    /// Always ignore any existing locks on the repository.
+    ///
+    /// ```
+    /// # use acid_store::repo::{OpenOptions, OpenMode, key::KeyRepo};
+    /// # use acid_store::store::MemoryConfig;
+    /// let mut repo: KeyRepo<String> = OpenOptions::new()
+    ///     .mode(OpenMode::Create)
+    ///     .locking(&[], |_| true)
+    ///     .open(&MemoryConfig::new())
+    ///     .unwrap();
+    /// ```
+    ///
+    /// [`Unlock::update_context`]: crate::repo::Unlock::update_context
+    pub fn locking(
+        &mut self,
+        context: &'a [u8],
+        handler: impl FnMut(&[u8]) -> bool + 'a,
+    ) -> &mut Self {
+        self.lock_context = context;
+        self.lock_handler = Box::new(handler);
         self
     }
 
@@ -249,18 +298,10 @@ impl OpenOptions {
     }
 
     /// Open the repository, failing if it doesn't exist.
-    fn open_repo<R: OpenRepo>(&self, mut store: impl DataStore + 'static) -> crate::Result<R> {
-        // Acquire a lock on the repository.
-        let repository_id = peek_info_store(&mut store)?.id();
-        let lock = REPO_LOCKS
-            .lock()
-            .unwrap()
-            .acquire_lock(repository_id)
-            .ok_or(crate::Error::Locked)?;
-
+    fn open_repo<R: OpenRepo>(&mut self, mut store: impl DataStore + 'static) -> crate::Result<R> {
         // Read the repository version to see if this is a compatible repository.
         let serialized_version = store
-            .read_block(VERSION_BLOCK_ID)
+            .read_block(BlockKey::Version)
             .map_err(crate::Error::Store)?
             .ok_or(crate::Error::NotFound)?;
         let version =
@@ -269,16 +310,15 @@ impl OpenOptions {
             return Err(crate::Error::UnsupportedRepo);
         }
 
-        // We read the metadata again after reading the UUID to prevent a race condition when
-        // acquiring the lock.
+        // Read the repository metadata from the super block.
         let serialized_metadata = store
-            .read_block(METADATA_BLOCK_ID)
+            .read_block(BlockKey::Super)
             .map_err(crate::Error::Store)?
             .ok_or(crate::Error::Corrupt)?;
         let metadata: RepoMetadata =
             from_read(serialized_metadata.as_slice()).map_err(|_| crate::Error::Corrupt)?;
 
-        let password = match self.password.clone() {
+        let password = match self.password {
             Some(password) if metadata.config.encryption != Encryption::None => Some(password),
             // Return an error if a password was required but not provided.
             None if metadata.config.encryption != Encryption::None => {
@@ -289,28 +329,32 @@ impl OpenOptions {
 
         // Decrypt the master key for the repository.
         let master_key = match password {
-            Some(password_bytes) => {
-                let user_key = EncryptionKey::derive(
-                    password_bytes.as_slice(),
-                    &metadata.salt,
-                    metadata.config.encryption.key_size(),
-                    metadata.config.memory_limit,
-                    metadata.config.operations_limit,
-                );
-                EncryptionKey::new(
-                    metadata
-                        .config
-                        .encryption
-                        .decrypt(&metadata.master_key, &user_key)
-                        .map_err(|_| crate::Error::Password)?,
-                )
-            }
+            Some(password_bytes) => metadata.decrypt_master_key(&password_bytes)?,
             None => EncryptionKey::new(Vec::new()),
         };
 
+        // Attempt to acquire a lock on the repository.
+        let lock_id = lock_store(
+            &mut store,
+            &metadata.config.encryption,
+            &master_key,
+            self.lock_context,
+            &mut self.lock_handler,
+        )?;
+
+        // We read the metadata again after acquiring a lock but before getting the header ID to
+        // avoid a race condition. We don't have to worry about decrypting the master encryption key
+        // again because the master encryption key should never change.
+        let serialized_metadata = store
+            .read_block(BlockKey::Super)
+            .map_err(crate::Error::Store)?
+            .ok_or(crate::Error::Corrupt)?;
+        let metadata: RepoMetadata =
+            from_read(serialized_metadata.as_slice()).map_err(|_| crate::Error::Corrupt)?;
+
         // Read, decrypt, decompress, and deserialize the repository header.
         let encrypted_header = store
-            .read_block(metadata.header_id)
+            .read_block(BlockKey::Header(metadata.header_id))
             .map_err(crate::Error::Store)?
             .ok_or(crate::Error::Corrupt)?;
         let compressed_header = metadata
@@ -339,7 +383,7 @@ impl OpenOptions {
             packs,
             transactions: LockTable::new(),
             master_key,
-            lock,
+            lock_id,
         }));
 
         let repo: KeyRepo<R::Key> = KeyRepo {
@@ -355,8 +399,11 @@ impl OpenOptions {
     }
 
     /// Create a new repository, failing if one already exists.
-    fn create_repo<R: OpenRepo>(&self, mut store: impl DataStore + 'static) -> crate::Result<R> {
-        let password = match self.password.clone() {
+    fn create_repo<R: OpenRepo>(
+        &mut self,
+        mut store: impl DataStore + 'static,
+    ) -> crate::Result<R> {
+        let password = match self.password {
             Some(password) if self.config.encryption != Encryption::None => Some(password),
             // Return an error if a password was required but not provided.
             None if self.config.encryption != Encryption::None => {
@@ -365,17 +412,9 @@ impl OpenOptions {
             _ => None,
         };
 
-        // Acquire an exclusive lock on the repository.
-        let id = Uuid::new_v4().into();
-        let lock = REPO_LOCKS
-            .lock()
-            .unwrap()
-            .acquire_lock(id)
-            .ok_or(crate::Error::AlreadyExists)?;
-
         // Check if the repository already exists.
         if store
-            .read_block(VERSION_BLOCK_ID)
+            .read_block(BlockKey::Version)
             .map_err(crate::Error::Store)?
             .is_some()
         {
@@ -388,6 +427,15 @@ impl OpenOptions {
             None => EncryptionKey::new(Vec::new()),
         };
 
+        // Attempt to acquire a lock on the data store.
+        let lock_id = lock_store(
+            &mut store,
+            &self.config.encryption,
+            &master_key,
+            self.lock_context,
+            &mut self.lock_handler,
+        )?;
+
         let salt = match password {
             Some(..) => KeySalt::generate(),
             None => KeySalt::empty(),
@@ -397,7 +445,7 @@ impl OpenOptions {
         let encrypted_master_key = match password {
             Some(password_bytes) => {
                 let user_key = EncryptionKey::derive(
-                    password_bytes.as_slice(),
+                    password_bytes,
                     &salt,
                     self.config.encryption.key_size(),
                     self.config.memory_limit,
@@ -428,13 +476,12 @@ impl OpenOptions {
             .encrypt(&compressed_header, &master_key);
         let header_id = Uuid::new_v4().into();
         store
-            .write_block(header_id, &encrypted_header)
+            .write_block(BlockKey::Header(header_id), &encrypted_header)
             .map_err(crate::Error::Store)?;
 
         // Create the repository metadata with the header block references.
         let metadata = RepoMetadata {
-            id,
-
+            id: Uuid::new_v4().into(),
             config: self.config.clone(),
             master_key: encrypted_master_key,
             salt,
@@ -444,13 +491,13 @@ impl OpenOptions {
         // Write the repository metadata.
         let serialized_metadata = to_vec(&metadata).expect("Could not serialize metadata.");
         store
-            .write_block(METADATA_BLOCK_ID, &serialized_metadata)
+            .write_block(BlockKey::Super, &serialized_metadata)
             .map_err(crate::Error::Store)?;
 
         // Write the repository version. We do this last because this signifies that the repository
         // is done being created.
         store
-            .write_block(VERSION_BLOCK_ID, VERSION_ID.as_bytes())
+            .write_block(BlockKey::Version, VERSION_ID.as_bytes())
             .map_err(crate::Error::Store)?;
 
         let Header {
@@ -467,7 +514,7 @@ impl OpenOptions {
             packs,
             transactions: LockTable::new(),
             master_key,
-            lock,
+            lock_id,
         }));
 
         let repo: KeyRepo<R::Key> = KeyRepo {
@@ -503,7 +550,7 @@ impl OpenOptions {
     /// - `Error::InvalidData`: Ciphertext verification failed.
     /// - `Error::Store`: An error occurred with the data store.
     /// - `Error::Io`: An I/O error occurred.
-    pub fn open<R, C>(&self, config: &C) -> crate::Result<R>
+    pub fn open<R, C>(&mut self, config: &C) -> crate::Result<R>
     where
         R: OpenRepo,
         C: OpenStore,
@@ -514,7 +561,7 @@ impl OpenOptions {
             OpenMode::Open => self.open_repo(store),
             OpenMode::Create => {
                 if store
-                    .read_block(VERSION_BLOCK_ID)
+                    .read_block(BlockKey::Version)
                     .map_err(crate::Error::Store)?
                     .is_some()
                 {
@@ -527,3 +574,15 @@ impl OpenOptions {
         }
     }
 }
+
+impl<'a> Debug for OpenOptions<'a> {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("OpenOptions")
+            .field("config", &self.config)
+            .field("mode", &self.mode)
+            .field("password", &self.password)
+            .field("instance", &self.instance)
+            .field("lock_context", &self.lock_context)
+            .finish_non_exhaustive()
+    }
+}
