@@ -19,13 +19,14 @@ use std::collections::HashSet;
 use hex_literal::hex;
 use serde::de::DeserializeOwned;
 use serde::Serialize;
+use static_assertions::assert_impl_all;
 use uuid::Uuid;
 
 use super::info::{KeyId, KeyIdTable, ObjectKey, RepoKey, RepoState, StateRestore};
 use super::iter::Keys;
 use crate::repo::{
     key::KeyRepo, Commit, InstanceId, Object, OpenRepo, RepoInfo, RepoStats, ResourceLimit,
-    RestoreSavepoint, Savepoint, VersionId,
+    RestoreSavepoint, Savepoint, Unlock, VersionId,
 };
 
 /// A low-level repository type which can be used to implement higher-level repository types
@@ -41,6 +42,8 @@ where
     state: State,
 }
 
+assert_impl_all!(StateRepo<()>: Send, Sync);
+
 impl<State> OpenRepo for StateRepo<State>
 where
     State: Serialize + DeserializeOwned + Default,
@@ -375,3 +378,24 @@ where
         true
     }
 }
+
+impl<State> Unlock for StateRepo<State>
+where
+    State: Serialize + DeserializeOwned + Default,
+{
+    fn unlock(&self) -> crate::Result<()> {
+        self.repo.unlock()
+    }
+
+    fn is_locked(&self) -> crate::Result<bool> {
+        self.repo.is_locked()
+    }
+
+    fn context(&self) -> crate::Result<Vec<u8>> {
+        self.repo.context()
+    }
+
+    fn update_context(&self, context: &[u8]) -> crate::Result<()> {
+        self.repo.update_context(context)
+    }
+}
