@@ -19,20 +19,22 @@
 use rstest_reuse::{self, *};
 use tempfile::TempDir;
 
+use acid_store::store::{
+    BlockId, BlockKey, BlockType, DataStore, MemoryConfig, MemoryStore, OpenStore,
+};
 #[cfg(feature = "store-directory")]
-use acid_store::store::DirectoryConfig;
+use acid_store::store::{DirectoryConfig, DirectoryStore};
 #[cfg(feature = "store-rclone")]
-use acid_store::store::RcloneConfig;
+use acid_store::store::{RcloneConfig, RcloneStore};
 #[cfg(feature = "store-redis")]
-use acid_store::store::RedisConfig;
-#[cfg(feature = "store-sqlite")]
-use acid_store::store::SqliteConfig;
-use acid_store::store::{BlockId, BlockKey, BlockType, DataStore, MemoryConfig, OpenStore};
+use acid_store::store::{RedisConfig, RedisStore};
 #[cfg(feature = "store-s3")]
-use acid_store::store::{S3Config, S3Credentials, S3Region};
+use acid_store::store::{S3Config, S3Credentials, S3Region, S3Store};
+#[cfg(feature = "store-sqlite")]
+use acid_store::store::{SqliteConfig, SqliteStore};
 #[cfg(feature = "store-sftp")]
 use {
-    acid_store::store::{SftpAuth, SftpConfig},
+    acid_store::store::{SftpAuth, SftpConfig, SftpStore},
     std::path::PathBuf,
 };
 
@@ -77,8 +79,32 @@ impl<T: DataStore> DataStore for WithTempDir<T> {
     }
 }
 
+impl<T: OpenStore> OpenStore for WithTempDir<T> {
+    type Store = T::Store;
+
+    fn open(&self) -> acid_store::Result<Self::Store> {
+        self.value.open()
+    }
+}
+
+pub fn memory_config() -> Box<dyn OpenStore<Store = MemoryStore>> {
+    Box::new(MemoryConfig::new())
+}
+
 pub fn memory_store() -> Box<dyn DataStore> {
-    Box::new(MemoryConfig::new().open().unwrap())
+    Box::new(memory_config().open().unwrap())
+}
+
+#[cfg(feature = "store-directory")]
+pub fn directory_config() -> Box<dyn OpenStore<Store = DirectoryStore>> {
+    let directory = tempfile::tempdir().unwrap();
+    let config = DirectoryConfig {
+        path: directory.as_ref().join("store"),
+    };
+    Box::new(WithTempDir {
+        directory,
+        value: config,
+    })
 }
 
 #[cfg(feature = "store-directory")]
@@ -95,6 +121,18 @@ pub fn directory_store() -> Box<dyn DataStore> {
     })
 }
 
+#[cfg(feature = "store-sqlite")]
+pub fn sqlite_config() -> Box<dyn OpenStore<Store = SqliteStore>> {
+    let directory = tempfile::tempdir().unwrap();
+    let config = SqliteConfig {
+        path: directory.as_ref().join("store.db"),
+    };
+    Box::new(WithTempDir {
+        directory,
+        value: config,
+    })
+}
+
 #[cfg(feature = "store-sqlite")]
 pub fn sqlite_store() -> Box<dyn DataStore> {
     let directory = tempfile::tempdir().unwrap();
@@ -110,17 +148,22 @@ pub fn sqlite_store() -> Box<dyn DataStore> {
 }
 
 #[cfg(feature = "store-redis")]
-pub fn redis_store() -> Box<dyn DataStore> {
+pub fn redis_config() -> Box<dyn OpenStore<Store = RedisStore>> {
     let url = dotenv::var("REDIS_URL").unwrap();
-    let config = RedisConfig::from_url(&url).unwrap();
+    Box::new(RedisConfig::from_url(&url).unwrap())
+}
+
+#[cfg(feature = "store-redis")]
+pub fn redis_store() -> Box<dyn DataStore> {
+    let config = redis_config();
     let mut store = config.open().unwrap();
     truncate_store(&mut store).unwrap();
     Box::new(store)
 }
 
 #[cfg(feature = "store-s3")]
-pub fn s3_store() -> Box<dyn DataStore> {
-    let config = S3Config {
+pub fn s3_config() -> Box<dyn OpenStore<Store = S3Store>> {
+    Box::new(S3Config {
         bucket: dotenv::var("S3_BUCKET").unwrap(),
         region: S3Region::from_name(&dotenv::var("S3_REGION").unwrap()).unwrap(),
         credentials: S3Credentials::Basic {
@@ -128,43 +171,71 @@ pub fn s3_store() -> Box<dyn DataStore> {
             secret_key: dotenv::var("S3_SECRET_KEY").unwrap(),
         },
         prefix: String::from("test"),
-    };
+    })
+}
+
+#[cfg(feature = "store-s3")]
+pub fn s3_store() -> Box<dyn DataStore> {
+    let config = s3_config();
     let mut store = config.open().unwrap();
     truncate_store(&mut store).unwrap();
     Box::new(store)
 }
 
 #[cfg(feature = "store-sftp")]
-pub fn sftp_store() -> Box<dyn DataStore> {
+pub fn sftp_config() -> Box<dyn OpenStore<Store = SftpStore>> {
     let sftp_server: String = dotenv::var("SFTP_SERVER").unwrap();
     let sftp_path: String = dotenv::var("SFTP_PATH").unwrap();
     let sftp_username: String = dotenv::var("SFTP_USERNAME").unwrap();
     let sftp_password: String = dotenv::var("SFTP_PASSWORD").unwrap();
 
-    let config = SftpConfig {
+    Box::new(SftpConfig {
         addr: sftp_server.parse().unwrap(),
         auth: SftpAuth::Password {
             username: sftp_username,
             password: sftp_password,
         },
         path: PathBuf::from(sftp_path),
-    };
-
+    })
+}
+#[cfg(feature = "store-sftp")]
+pub fn sftp_store() -> Box<dyn DataStore> {
+    let config = sftp_config();
     let mut store = config.open().unwrap();
     truncate_store(&mut store).unwrap();
     Box::new(store)
 }
 
 #[cfg(feature = "store-rclone")]
-pub fn rclone_store() -> Box<dyn DataStore> {
-    let config = RcloneConfig {
+pub fn rclone_config() -> Box<dyn OpenStore<Store = RcloneStore>> {
+    Box::new(RcloneConfig {
         config: dotenv::var("RCLONE_REMOTE").unwrap(),
-    };
+    })
+}
+
+#[cfg(feature = "store-rclone")]
+pub fn rclone_store() -> Box<dyn DataStore> {
+    let config = rclone_config();
     let mut store = config.open().unwrap();
     truncate_store(&mut store).unwrap();
     Box::new(store)
 }
 
+/// A parameterized test template which provides a data store config of each type.
+///
+/// The generates tests are serialized to avoid race conditions with concurrent access to shared
+/// resources.
+#[template]
+#[rstest]
+#[case::store_memory(memory_config())]
+#[cfg_attr(feature = "store-directory", case::store_directory(directory_config()))]
+#[cfg_attr(feature = "store-sqlite", case::store_sqlilte(sqlite_config()))]
+#[cfg_attr(feature = "store-redis", case::store_redis(redis_config()))]
+#[cfg_attr(feature = "store-s3", case::store_s3(s3_config()))]
+#[cfg_attr(feature = "store-sftp", case::store_sftp(sftp_config()))]
+#[cfg_attr(feature = "store-rclone", case::store_rclone(rclone_config()))]
+pub fn data_configs(#[case] config: Box<dyn OpenStore>) {}
+
 /// A parameterized test template which provides a data store of each type.
 ///
 /// The generates tests are serialized to avoid race conditions with concurrent access to shared
